import argparse
from pathlib import Path

from plugins import PluginManager


def main() -> None:
    parser = argparse.ArgumentParser(description="Butterfly application")
    parser.add_argument(
        "--plugin-dir", type=Path, help="Directory to search for plugins"
    )
    parser.add_argument(
        "--list-plugins", action="store_true", help="List available plugins and exit"
    )
    parser.add_argument(
        "--plugins",
        nargs="*",
        help="Specify plugin to use (all)",
    )
    parser.add_argument(
        "--method",
        help="Method to execute on the plugin",
        choices=["validate", "execute"],
    )
    parser.add_argument("--message", type=str, help="text content to post")
    parser.add_argument("--debug", action="store_true", help="Enable debug mode")

    args = parser.parse_args()

    if args.plugin_dir:
        pm = PluginManager(plugin_dir=args.plugin_dir).discover_plugins()
    else:
        pm = PluginManager().discover_plugins()

    if not args.method or args.list_plugins:
        print("Available plugins:")
        for plugin_name in pm.plugins:
            print(f"  - {plugin_name}")
        return

    for plugin in args.plugins or [""]:
        if args.method == "execute" and not args.message:
            print("You need to provide content to post")
            break
        pm._run_method(plugin, args.method, args.message)


if __name__ == "__main__":
    main()



import sys
from pathlib import Path

import pytest

from plugins import PluginManager


@pytest.fixture
def manager() -> PluginManager:
    current_dir = Path(__file__).resolve().parent
    sys.path.insert(0, str(current_dir))
    return PluginManager(plugin_dir=current_dir / "plugins").discover_plugins()


def test_plugin_manager(manager: PluginManager) -> None:
    assert manager._run_method("hello", "execute", ("arg1",), key="value")
    assert manager._run_method("hello", "validate", ("arg1",), key="value")
    assert manager._run_method("hello", "doesnt_exist") is False
    assert manager._run_method("doesnt_exist", "404") is False


def test_supported_plugins() -> None:
    supported_plugins = PluginManager().discover_plugins()
    for plugin in supported_plugins.get_plugins():
        assert supported_plugins._run_method(plugin, "get_name")



import importlib
import inspect
from pathlib import Path
from typing import Any, Protocol, Self, runtime_checkable


@runtime_checkable
class Plugin(Protocol):
    def get_name(self) -> str: ...
    def validate(self, *args: tuple[Any], **kwargs: dict[str, Any]) -> bool: ...
    def execute(self, *args: tuple[Any], **kwargs: dict[str, Any]) -> bool: ...


class PluginManager:
    def __init__(self, plugin_dir: Path = Path("plugins")) -> None:
        self.plugin_dir = plugin_dir
        self.plugins: dict[str, Plugin] = {}

    def discover_plugins(self) -> Self:
        for file_path in self.plugin_dir.glob("*.py"):
            if file_path.name != "__init__.py":
                self._load_plugin(file_path)
        print(f"Loaded {len(self.plugins)} plugins")
        return self

    def _load_plugin(self, module_file: Path) -> None:
        try:
            full_module = self.get_full_module(module_file)
            module = importlib.import_module(full_module)
            for _, klass in inspect.getmembers(module, inspect.isclass):
                if issubclass(klass, Plugin):
                    plugin_inst = klass()
                    plugin_name = plugin_inst.get_name()
                    self.plugins[plugin_name] = plugin_inst
                    print(f"Loaded plugin: {plugin_name}")
        except Exception as e:
            print(f"Error loading plugin module {module_file}: {e}")

    def get_plugins(self) -> list[str]:
        return list(self.plugins.keys())

    @staticmethod
    def get_full_module(module_file: Path, marker_file: str = "pyproject.toml") -> str:
        path = []
        path.append(module_file.stem)
        current = module_file.parent
        while current != current.parent:
            if (current / marker_file).exists():
                break
            path.append(current.name)
            current = current.parent

        return ".".join(reversed(path))

    def _run_method(
        self,
        plugin_name: str | None,
        method: str,
        *args: Any,
        **kwargs: Any,
    ) -> bool:
        success = True
        plugins = [plugin_name] if plugin_name else self.get_plugins()
        for plugin in plugins:
            try:
                plugin_method = getattr(self.plugins[plugin], method)
                plugin_method(*args, **kwargs)
            except KeyError:
                print(f"Plugin not found {plugin_name}")
                success = False
            except Exception as e:
                print(f"Failed to execute {plugin_name}.{method}(): {e}")
                success = False
        return success



from typing import Any

import requests

from interfaces import auth


class Twitter:
    def __init__(self, auth: auth.OAuth1 = auth.OAuth1()) -> None:
        self.name = "twitter"
        self.auth = auth
        self.api_base_url = "https://api.twitter.com/2"

    def get_name(self) -> str:
        return self.name

    def get_user_info(self) -> bool | Any:
        """
        Test the API connection by making a simple request.
        Twitter API v2 endpoint for getting user details
        """
        # Using the /2/users/me endpoint which requires a valid token
        endpoint = f"{self.api_base_url}/users/me"

        try:
            response = requests.get(endpoint, auth=self.auth, timeout=10)

            # Check if the request was successful
            if response.status_code == 200:
                print("Twitter API connection successful!")
                return response.json()
            else:
                error_msg = (
                    response.json()
                    .get("errors", [{}])[0]
                    .get("message", "Unknown error")
                )
                print(
                    f"Twitter API connection failed: {response.status_code} - {error_msg}"
                )
                return False

        except requests.RequestException as e:
            print(f"Request to Twitter API failed: {e}")
            return False

    def authorize(self, *args: tuple[Any]) -> bool:
        if self.auth:
            return self.auth.authorize()
        return False

    def validate(self, *args: tuple[Any], **kwargs: dict[str, Any]) -> bool:
        if not self.authorize():
            print("Invalid Credentials")
            return False
        return self.get_user_info()

    def execute(self, *args: tuple[Any], **kwargs: dict[str, Any]) -> bool:
        endpoint = f"{self.api_base_url}/tweets"

        try:
            response = requests.post(
                endpoint, auth=self.auth, json={"text": args[0]}, timeout=10
            )

            # Check if the request was successful
            if response.status_code == 201:
                return True
            else:
                error_msg = (
                    response.json()
                    .get("errors", [{}])[0]
                    .get("message", "Unknown error")
                )
                print(
                    f"Twitter API connection failed: {response.status_code} - {error_msg}"
                )
                return False

        except requests.RequestException as e:
            print(f"Request to Twitter API failed: {e}")
            return False



import os
import re
from typing import Any

from atproto import client_utils

from interfaces.auth import BlueSkyAuth


class BlueSky:
    def __init__(
        self,
        auth: BlueSkyAuth = BlueSkyAuth(
            username=os.environ["BSKY_USERNAME"], password=os.environ["BSKY_PASSWORD"]
        ),
    ) -> None:
        self.name = "bluesky"
        self.auth = auth

    def get_name(self) -> str:
        return self.name

    def authorize(self, *args: tuple[Any]) -> bool:
        return bool(self.auth.username and self.auth.password)

    def validate(self, *args: tuple[Any], **kwargs: dict[str, Any]) -> bool:
        if not self.authorize():
            print("Invalid Credentials")
            return False
        return True

    def execute(self, *args: tuple[Any], **kwargs: dict[str, Any]) -> bool:
        if not self.authorize():
            print("Invalid Credentials")
            return False
        if not args and args[0]:
            print("Invalid Text")
            return False

        text = str(args[0])
        text, urls = self.extract_urls(text)
        builder = client_utils.TextBuilder()
        builder.text(text)
        for url in urls:
            builder.link(url, url)
        client = self.auth.get_client()
        client.send_post(builder)
        return True

    def extract_urls(self, text: str) -> tuple[str, list[str]]:
        # Regular expression pattern for matching URLs
        url_pattern = r"https?://(?:[-\w.]|(?:%[\da-fA-F]{2}))+"

        # Find all URLs in the text
        urls = re.findall(url_pattern, text)

        # Remove all URLs from the text
        text_without_urls = re.sub(url_pattern, "", text)

        return text_without_urls, urls



import os
from typing import Any

import requests

from interfaces.auth import BearerAuth


class Mastodon:
    def __init__(
        self,
        auth: BearerAuth = BearerAuth(access_token=os.environ["MASTODON_BEARER_TOKEN"]),
    ) -> None:
        self.name = "mastodon"
        self.auth = auth
        self.api_base_url = "https://techhub.social/api"

    def get_name(self) -> str:
        return self.name

    def get_user_info(self) -> bool | Any:
        endpoint = f"{self.api_base_url}/v1/accounts/verify_credentials"
        try:
            response = requests.get(endpoint, headers=self.auth.header, timeout=10)
            # Check if the request was successful
            if response.status_code == 200:
                print("Mastodon API connection successful!")
                return response.json()
            else:
                error_msg = (
                    response.json()
                    .get("errors", [{}])[0]
                    .get("message", "Unknown error")
                )
                print(
                    f"Mastodon API connection failed: {response.status_code} - {error_msg}"
                )
                return False

        except requests.RequestException as e:
            print(f"Request to Mastodon API failed: {e}")
            return False

    def authorize(self, *args: tuple[Any]) -> bool:
        return bool(self.auth)

    def validate(self, *args: tuple[Any], **kwargs: dict[str, Any]) -> bool:
        if not self.auth:
            print("Invalid Credentials")
            return False
        return self.get_user_info()

    def execute(self, *args: tuple[Any], **kwargs: dict[str, Any]) -> bool:
        endpoint = f"{self.api_base_url}/v1/statuses"

        try:
            response = requests.post(
                endpoint,
                headers=self.auth.header,
                data={"status": args[0]},
                timeout=10,
            )

            # Check if the request was successful
            if response.status_code == 200:
                return True
            else:
                error_msg = (
                    response.json()
                    .get("errors", [{}])[0]
                    .get("message", "Unknown error")
                )
                print(
                    f"Mastodon API connection failed: {response.status_code} - {error_msg}"
                )
                return False

        except requests.RequestException as e:
            print(f"Request to Mastodon API failed: {e}")
            return False



import time
from pathlib import Path
from typing import Generator, Self

import pytest

from interfaces.auth import BearerAuth, BlueSkyAuth, HashableMixin, SessionCacheMixin


class GrantedAuth:
    def authorize(self: Self) -> bool:
        return True


@pytest.fixture
def auth() -> GrantedAuth:
    return GrantedAuth()


def test_auth(auth: GrantedAuth) -> None:
    assert auth.authorize()


class FSCache(HashableMixin, SessionCacheMixin):
    pass


@pytest.fixture(scope="function")
def fake_session_cache() -> Generator[FSCache, None, None]:
    fname = ".test-session"
    secs = 200.0

    yield FSCache()._override_defaults(fname, secs)

    Path(fname).unlink(missing_ok=True)  # might delete because stale


@pytest.mark.parametrize(
    "access_token,is_valid,header_output",
    (
        ("", False, {"Authorization": "Bearer "}),
        (None, False, {"Authorization": "Bearer None"}),
        ("foobar", True, {"Authorization": "Bearer foobar"}),
    ),
)
def test_bearer_auth(
    access_token: str, is_valid: bool, header_output: dict[str, str]
) -> None:
    bt = BearerAuth(access_token=access_token)
    assert bool(bt) == is_valid
    if not bt:
        with pytest.raises(ValueError):
            assert bt.header == header_output
    else:
        assert bt.header == header_output


def test_session_cache_retervial(fake_session_cache: FSCache) -> None:
    assert not fake_session_cache.get_session()
    fake_session_cache.save_session("foobar")
    assert fake_session_cache.get_session() == "foobar"
    fake_session_cache.save_session("barfoo")
    assert fake_session_cache.get_session() != "foobar"
    assert fake_session_cache.get_session() == "barfoo"


def test_session_cache_file(fake_session_cache: FSCache) -> None:
    fake_session_cache.save_session("foobar")
    assert fake_session_cache.get_session() == "foobar"


def test_session_cache_stale_file(fake_session_cache: FSCache) -> None:
    fake_session_cache.save_session("foobar")
    assert fake_session_cache.get_session() == "foobar"

    fake_session_cache._update_session_time(
        time.time() - fake_session_cache.stale_seconds
    )
    assert not fake_session_cache.get_session()


def test_bluesky_hashable() -> None:
    ba = BlueSkyAuth("foo", "bar")
    assert hash(ba)

    ba.save_session("hello")
    assert ba.get_session() == "hello"
    Path(ba.session_filename).unlink(missing_ok=True)



import base64
import hashlib
import hmac
import os
import shelve
import time
import urllib.parse
from dataclasses import dataclass
from typing import Any, Protocol, Self

from atproto import Client
from requests.auth import AuthBase


class Auth(Protocol):
    def authorize(self, *args: tuple[Any]) -> bool: ...


class UsernameAuth:
    def __init__(self, username: str, password: str):
        self.username, self.password = username, password


class HashableMixin:
    def __hash__(self) -> int:
        return hash("".join(map(str, vars(self).values())))

    def __eq__(self, obj: object) -> bool:
        if isinstance(obj, type(self)):
            raise TypeError()
        return all(getattr(self, att) == getattr(obj, att) for att in vars(self).keys())


class SessionCacheMixin:
    session_filename: str = ".session"
    stale_seconds: float = 5 * 60.0
    UPDATED_AT = "UPDATED_AT"

    def get_session(self) -> str | None:
        with shelve.open(self.session_filename) as db:
            last_update = db.get(self.UPDATED_AT, time.time())
            if (time.time() - last_update) > self.stale_seconds:
                return None
            return db.get(str(hash(self)))

    def save_session(self, session: str) -> None:
        with shelve.open(self.session_filename) as db:
            db[str(hash(self))] = session
            db[self.UPDATED_AT] = time.time()

    def _override_defaults(self, fname: str, secs: float) -> Self:
        self.session_filename, self.stale_seconds = fname, secs
        return self

    def _update_session_time(self, secs: float) -> None:
        with shelve.open(self.session_filename) as db:
            db[self.UPDATED_AT] = secs


class BlueSkyAuth(UsernameAuth, HashableMixin, SessionCacheMixin):
    def __init__(
        self,
        username: str,
        password: str,
    ):
        super().__init__(username, password)

    def get_client(self) -> Client:
        client = Client()
        if session := self.get_session():
            client.login(session_string=session)
        else:
            client.login(self.username, self.password)
            self.save_session(client.export_session_string())
        return client


@dataclass
class BearerAuth:
    key: str = "Authorization"
    access_token: str = ""

    @property
    def header(self) -> dict[str, str]:
        if not self:
            raise ValueError("Invalid BearerToken")
        return {self.key: f"Bearer {self.access_token}"}

    def __bool__(self) -> bool:
        return bool(self.key and self.access_token)


class OAuth1(AuthBase):
    """
    Twitter OAuth 1.0a implementation for use with the requests library.
    """

    def __init__(
        self,
        consumer_key: str = os.environ.get("TWITTER_CONSUMER_KEY", ""),
        consumer_secret: str = os.environ.get("TWITTER_CONSUMER_SECRET", ""),
        access_token: str = os.environ.get("TWITTER_ACCESS_TOKEN", ""),
        access_token_secret: str = os.environ.get("TWITTER_ACCESS_TOKEN_SECRET", ""),
    ):
        self.consumer_key = consumer_key
        self.consumer_secret = consumer_secret
        self.access_token = access_token
        self.access_token_secret = access_token_secret

    def __call__(self, request: Any) -> Any:
        # Add OAuth parameters
        oauth_params = {
            "oauth_consumer_key": self.consumer_key,
            "oauth_nonce": self._generate_nonce(),
            "oauth_signature_method": "HMAC-SHA1",
            "oauth_timestamp": str(int(time.time())),
            "oauth_token": self.access_token,
            "oauth_version": "1.0",
        }

        # Get request method and URL
        method = request.method
        url = request.url
        url_parts = urllib.parse.urlparse(url)
        base_url = f"{url_parts.scheme}://{url_parts.netloc}{url_parts.path}"

        # Extract and encode request parameters
        params: dict[str, str | None] = {}
        if url_parts.query:
            query_params = urllib.parse.parse_qsl(url_parts.query)
            params.update(query_params)

        # Add body parameters if this is a form request
        if request.body and isinstance(request.body, bytes):
            content_type = request.headers.get("Content-Type", "")
            if "application/x-www-form-urlencoded" in content_type:
                body_params = urllib.parse.parse_qsl(request.body.decode("utf-8"))
                params.update(body_params)

        # Add OAuth parameters to the signature base
        params.update(oauth_params)

        # Create signature base string
        param_string = "&".join(
            f"{urllib.parse.quote(key, safe='')}={urllib.parse.quote(str(value), safe='')}"
            for key, value in sorted(params.items())
        )
        signature_base = (
            f"{method}&{urllib.parse.quote(base_url, safe='')}"
            f"&{urllib.parse.quote(param_string, safe='')}"
        )

        # Create signing key
        signing_key = (
            f"{urllib.parse.quote(self.consumer_secret, safe='')}"
            f"&{urllib.parse.quote(self.access_token_secret, safe='')}"
        )

        # Calculate signature
        signature = base64.b64encode(
            hmac.new(
                signing_key.encode("utf-8"),
                signature_base.encode("utf-8"),
                hashlib.sha1,
            ).digest()
        ).decode("utf-8")

        # Add signature to OAuth parameters
        oauth_params["oauth_signature"] = signature

        # Create Authorization header
        auth_header = "OAuth " + ", ".join(
            f'{urllib.parse.quote(key, safe="")}="{urllib.parse.quote(value, safe="")}"'
            for key, value in sorted(oauth_params.items())
        )

        request.headers["Authorization"] = auth_header
        return request

    def _generate_nonce(self) -> str:
        return hashlib.sha1(str(time.time()).encode("utf-8")).hexdigest()

    def authorize(self, *_: tuple[Any]) -> bool:
        return all(
            [
                self.consumer_key,
                self.consumer_secret,
                self.access_token,
                self.access_token_secret,
            ]
        )
